@@ -36,7 +36,7 @@ const (
 // Handler handles dns query.
 type Handler interface {
 	// ServeDNS handles r and writes response to w.
-	// meta may be nil.
+	// Implements must not keep and use req after the ServeDNS returned.
 	ServeDNS(ctx context.Context, req []byte, w ResponseWriter, meta *handler.RequestMeta)
 }
 
@@ -121,25 +121,34 @@ func (h *DefaultHandler) ServeDNS(ctx context.Context, req []byte, w ResponseWri
 		h.logger().Debug("entry returned", qCtx.InfoField(), zap.Stringer("status", qCtx.Status()))
 	}
 
-	var rm *dns.Msg
+	var respMsg *dns.Msg
 	if err != nil || qCtx.Status() == handler.ContextStatusServerFailed {
-		rm = new(dns.Msg)
-		rm.SetReply(reqMsg)
-		rm.Rcode = dns.RcodeServerFailure
+		respMsg = new(dns.Msg)
+		respMsg.SetReply(reqMsg)
+		respMsg.Rcode = dns.RcodeServerFailure
 	} else {
-		rm = qCtx.R()
+		respMsg = qCtx.R()
 	}
 
-	if rm != nil {
+	if respMsg != nil {
 		if h.RecursionAvailable {
-			rm.RecursionAvailable = true
+			respMsg.RecursionAvailable = true
 		}
-		raw, buf, err := pool.PackBuffer(rm)
+		if meta.FromUDP {
+			udpSize := dns.MinMsgSize
+			if opt := reqMsg.IsEdns0(); opt != nil {
+				if es := int(opt.UDPSize()); es > udpSize {
+					udpSize = es
+				}
+			}
+			respMsg.Truncate(udpSize)
+		}
+		raw, buf, err := pool.PackBuffer(respMsg)
 		if err != nil {
 			h.logger().Warn("failed to pack response message", qCtx.InfoField(), zap.Error(err))
 			return
 		}
-		defer pool.ReleaseBuf(buf)
+		defer buf.Release()
 
 		if _, err := w.Write(raw); err != nil {
 			h.logger().Warn("failed to write response", qCtx.InfoField(), zap.Error(err))
