@@ -20,8 +20,9 @@ package sequence
 import (
 	"context"
 	"fmt"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/handler"
-	"github.com/IrineSistiana/mosdns/v2/dispatcher/pkg/executable_seq"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/handler"
+	"github.com/IrineSistiana/mosdns/v3/dispatcher/pkg/executable_seq"
+	"sync"
 )
 
 const PluginType = "sequence"
@@ -29,7 +30,10 @@ const PluginType = "sequence"
 func init() {
 	handler.RegInitFunc(PluginType, Init, func() interface{} { return new(Args) })
 
-	handler.MustRegPlugin(&end{BP: handler.NewBP("_end", PluginType)}, true)
+	handler.MustRegPlugin(&_return{BP: handler.NewBP("_return", PluginType)})
+
+	// TODO: Remove deprecated.
+	handler.MustRegPlugin(&_end{BP: handler.NewBP("_end", PluginType)})
 }
 
 var _ handler.ExecutablePlugin = (*sequence)(nil)
@@ -68,14 +72,26 @@ func (s *sequence) Exec(ctx context.Context, qCtx *handler.Context, next handler
 	return handler.ExecChainNode(ctx, qCtx, next)
 }
 
-var _ handler.ExecutablePlugin = (*end)(nil)
+var _ handler.ExecutablePlugin = (*_end)(nil)
+var _ handler.ExecutablePlugin = (*_return)(nil)
 
-// end is a no-op handler.Executable.
-// It can be used as a stop sign in the handler.ExecutableChainNode.
-type end struct {
+// TODO: Remove deprecated.
+type _end struct {
+	*handler.BP
+	warnOnce sync.Once
+}
+
+func (n *_end) Exec(_ context.Context, _ *handler.Context, _ handler.ExecutableChainNode) error {
+	n.warnOnce.Do(func() {
+		n.L().Warn("_end is deprecated, use _return instead")
+	})
+	return nil
+}
+
+type _return struct {
 	*handler.BP
 }
 
-func (n *end) Exec(ctx context.Context, qCtx *handler.Context, next handler.ExecutableChainNode) error {
+func (n *_return) Exec(_ context.Context, _ *handler.Context, _ handler.ExecutableChainNode) error {
 	return nil
 }
